unit Graph2d;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TPoint         = array [1..2] of Double;
  TArrayOfPoints = array [0..800] of TPoint;

  TDraw2D = class(TForm)
    Image1: TImage;
  private
    { Private declarations }
  public
    { Public declarations }
    Red,Green,Blue:Byte;
    Unit_X,Unit_Y:Double;
    O,S          :TPoint;
    x1,x2,y1,y2  :Double;
    x,y,hx,hy    :Double;
    Q            :TArrayOfPoints;
    ShowAxes  :Boolean;
    procedure ClearImage;
    procedure Scale(x1,x2,y1,y2:Double);
    function  XCoor(x:Double):integer;
    function  YCoor(y:Double):integer;
    procedure PutPixel(X:TPoint;Color:DWord);
    procedure MoveTo(X:TPoint);
    procedure LineTo(X:TPoint;Color:DWord);
    procedure Line(X,Y:TPoint;Color:DWord);
    Procedure XAxis(x1,x2,y:Double;Color:DWord);
    Procedure YAxis(y1,y2,x:Double;Color:DWord);
    Procedure XScale(x1,x2,y:Double;Color:DWord);
    procedure YScale(y1,y2,x:Double;Color:DWord);
    Procedure PolyLine(Q:TArrayOfPoints;n:Word;Color:DWord);
  end;

var
  Draw2D: TDraw2D;

implementation

uses ControlPanel;

{$R *.DFM}

 procedure TDraw2D.ClearImage;
 var i:Integer;
 begin
  with Image1.Canvas do
    begin Pen.Color:=clWhite;Brush.Color:=clWhite;end;
  with Image1 do Canvas.Rectangle(0,0,Width-1,Height-1);
 end;

Procedure TDraw2D.Scale(x1,x2,y1,y2:Double);
  begin
     Unit_X:= Image1.Width/(x2-x1);
     Unit_Y:= Image1.Height/(y2-y1);
     O[1]  := -x1*Unit_X;
     O[2]  :=  y2*Unit_Y;
  end;

  function  TDraw2D.XCoor(x:Double):integer;
  begin
    XCoor:=trunc(O[1]+Unit_X*x);
  end;

  function  TDraw2D.YCoor(y:Double):integer;
  begin
     YCoor:=trunc(O[2]-Unit_Y*y);
  end;

  procedure TDraw2D.PutPixel(X:TPoint;Color:DWord);
  begin
    Image1.Canvas.Pixels[XCoor(X[1]),YCoor(X[2])]:=Color;
  end;

  procedure TDraw2D.MoveTo(X:TPoint);
  begin
    Image1.Canvas.MoveTo(XCoor(X[1]),YCoor(X[2]));
  end;

  procedure TDraw2D.LineTo(X:TPoint;Color:DWord);
  begin
   With Image1.Canvas do
     begin
       Pen.Color:=Color;
       LineTo(XCoor(X[1]),YCoor(X[2]))
     end;
  end;

  procedure TDraw2D.Line(X,Y:TPoint;Color:DWord);
  begin
    With Image1.Canvas do
     begin
       Pen.Color:=Color;
       MoveTo(XCoor(X[1]),YCoor(X[2]));
       LineTo(XCoor(Y[1]),YCoor(Y[2]));
     end;
  end;

  Procedure TDraw2D.XAxis(x1,x2,y:Double;Color:DWord);
  var A,B:TPoint;
  begin
    A[1]:=x1;A[2]:=y;B[1]:=x2;B[2]:=y;
    Line(A,B,Color);
  end;

  Procedure TDraw2D.YAxis(y1,y2,x:Double;Color:DWord);
  var A,B:TPoint;
  begin
    A[1]:=x;A[2]:=y1;B[1]:=x;B[2]:=y2;
    Line(A,B,Color);
  end;

  procedure TDraw2D.XScale(x1,x2,y:Double;Color:DWord);
  var AxisUnit        :Real;
       Text            :String;
       Lx,Exponent     :Integer;

   begin
   Image1.Canvas.Font.Color:=Color;
   AxisUnit:=1E30;Lx:=0;Exponent:=30;
   While (x2-x1)/AxisUnit<5 do
    begin
       AxisUnit:=AxisUnit/10;Exponent:=Exponent-1;
    end;
   Lx:=0;
   if x1<0 then
     Repeat Lx:=Lx-1 Until AxisUnit*(Lx-1)<x1
           else
     Repeat Lx:=Lx+1 Until AxisUnit*(Lx+1)>x1;
   With Image1.Canvas do
   repeat
      MoveTo(XCoor(AxisUnit*Lx),YCoor(y)-3);
      LineTo(XCoor(AxisUnit*Lx),YCoor(y)+3);
      Str(Lx,Text);
      TextOut(XCoor(AxisUnit*Lx),YCoor(y)+5,Text);
      Lx:=Lx+1;
   until AxisUnit*(Lx+1)>x2;
   Str(Exponent,Text);
   Image1.Canvas.TextOut(XCoor(x2)-30,YCoor(y)+5,'*10');
   Image1.Canvas.TextOut(XCoor(x2)-10,YCoor(Y),Text);
   end;

procedure TDraw2D.YScale(y1,y2,x:Double;Color:DWord);
   var AxisUnit        :Real;
       Text            :String;
       Ly,Exponent     :Integer;
   begin
   Image1.Canvas.Font.Color:=Color;
   AxisUnit:=1E30;Ly:=0;Exponent:=30;
   While (y2-y1)/AxisUnit<5 do
    begin
       AxisUnit:=AxisUnit/10;Exponent:=Exponent-1;
    end;
   Ly:=0;
   if y1<0 then
     Repeat Ly:=Ly-1 Until AxisUnit*(Ly-1)<y1
           else
     Repeat Ly:=Ly+1 Until AxisUnit*(Ly+1)>y1;
   With Image1.Canvas do
   repeat
      MoveTo(XCoor(x)-3,YCoor(AxisUnit*Ly));
      LineTo(XCoor(x)+3,YCoor(AxisUnit*Ly));
      Str(Ly,Text);
      TextOut(XCoor(x)-15,YCoor(AxisUnit*Ly)-7,Text);
      Ly:=Ly+1;
   until AxisUnit*(Ly+1)>y2;
   Str(Exponent,Text);
   Image1.Canvas.TextOut(XCoor(x)-30,YCoor(y2)+10,'*10');
   Image1.Canvas.TextOut(XCoor(x)-10,YCoor(y2)+5,Text);
   end;

Procedure TDraw2D.PolyLine(Q:TArrayOfPoints;n:Word;Color:DWord);
var i:Word;
begin
  MoveTo(Q[0]);
  For i:=1 to n-1 do LineTo(Q[i],Color);
end;
end.
